#include "msgServer.hh"

using namespace std;

msgServer::msgServer(unsigned int p, unsigned int inact) {
	this->currentPort = p;
	this->inactivity_time = inact;
	this->isRunning = iserv.createServer(this->currentPort, MAX_CLIENT_CONNECTIONS);
	isMessagesWaiting = false;
	if(this->isRunning) {
		iserv.setInactivityTime(this->inactivity_time);
		cout << "----------------------" << endl;
		cout << "msgconnector Server in now running on port localhost:" << this->currentPort << endl;
		cout << " - max authorized connections: " << MAX_CLIENT_CONNECTIONS << endl;
		cout << " - max inactivity time: " << this->inactivity_time << endl;
		cout << "----------------------" << endl;		
	}
}

void msgServer::setInactivityTime(unsigned int inact) {
	this->inactivity_time = inact;
	if(this->isRunning) {
		iserv.setInactivityTime(this->inactivity_time);
	}
}

void msgServer::addConnection(int clientId, string clientName) {
	clients.insert(std::make_pair(clientId, clientName));
	cout << clientName << " is now connected (id=" << clientId << ")" << endl;
}

void msgServer::removeConnection(int c) {
	if(clients.find(c)!=clients.end()){
		string n = clients[c];
		clients.erase(clients.find(c));
		cout << n << " is disconnected (id=" << c << ")" << endl;
	}
	iserv.removeClientConnection(c);
}

void msgServer::manageIncomingMessage(int c, message m) {
	string dest = m.getStringHead();
	string content = m.getStringBody();
	if(dest == DEFAULT_SERVER_NAME) { // command
		if(strncmp(content.c_str(), REQUEST_TEST_CONNECTED, strlen(REQUEST_TEST_CONNECTED)) == 0) {
			size_t lwho = strlen(content.c_str()) - strlen(REQUEST_TEST_CONNECTED) -1;
			char who[lwho+1];
			memcpy(who, content.c_str()+strlen(REQUEST_TEST_CONNECTED)+1, lwho+1);
			bool cwho = (find(clients.begin(), clients.end(), who) != clients.end());
			message res;
			if(cwho) res.encode(DEFAULT_SERVER_NAME, REQUEST_TRUE);
			else res.encode(DEFAULT_SERVER_NAME, REQUEST_FALSE);
			iserv.send(c, res);
		}
		else this->addConnection(c, content);
	}
	else { // transfer
		for(clientList::iterator it=clients.begin(); it!=clients.end(); ++it){
			if(it->second==dest){
				string from = clients[c];
				int client = it->first;
				message temp;
				temp.encode(from.c_str(), content.c_str());
				if(!iserv.send(client, temp)) std::cout<< "error in send" << endl;
			}
		}
	}
}

void msgServer::run() {
	while(this->isRunning) {
		vector<pair<int, int> > events = iserv.waitReadEvents();
		for(size_t i = 0 ; i < events.size() ; i++) {
			pair<int, int> event = events[i];
			if(event.first == iserv.READ_EVENT && event.second >= -1) {
				if(iserv.isMainSocket(event.second)) iserv.addClientConnection(); 
				else {
					if(event.first == iserv.ERROR_EVENT) this->removeConnection(event.second);
					else { 
						pair<int, message> recpt = iserv.receive(event.second);
						if(recpt.first == iserv.STAT_ERR) this->removeConnection(event.second);
						else this->manageIncomingMessage(event.second, recpt.second);
					}
				}
			}
			if(event.second == 0) this->shutdown();
		}
	}
}

void msgServer::shutdown() {
	if(this->isRunning) {
		this->isRunning = false;
		iserv.stopServer();
	}
}

msgServer::~msgServer() {
	cout << "shuting down server... " << endl;
	this->shutdown();
}

